//==========================================================================
// File: rhr.c
// Description: An implementation of right hand routing
// Author: Pat Morin
// Created: Wed Jan 20 20:13:58 EST 1999
//==========================================================================
#include "rhr.h"

//==========================================================================
// Find the right hand routing path between all pairs of vertices in G.
// Complexity: O(n^3)
// Precondition: G is a convex subdivision
// Implementation: 
// Straightforward, although it actually implements the left hand
// rule, for historical reasons.
//========================================================================
void ALL_PAIRS_RIGHTHAND_PATHS(graph &G, node_array<point>& pos, 
			       node_matrix<double>& dist)
{
  node u, v;
  forall_nodes(v, G) {
    forall_nodes(u, G) {
      dist[u][v] = 0.0;
      if (u != v) {
	// find our first step, the first edge to the left of the
	// directed line through u and v
	edge e = G.first_adj_edge(u);
	while ((G.target(e) != v)
	       && right_turn(pos[u], pos[G.target(e)], pos[v]))
	  e = G.cyclic_adj_succ(e);
	while ((G.target(e) != v)
	       && left_turn(pos[u], pos[G.target(e)], pos[v]))
	  e = G.cyclic_adj_succ(e);
	dist[u][v] += pos[G.source(e)].distance(pos[G.target(e)]);
	e = G.cyclic_adj_succ(G.reversal(e));

	// from now on we will assume we are standing at G.source(e)
	while (G.source(e) != v) {
	  while (G.target(e) != v 
		 && right_turn(pos[u], pos[v], pos[G.target(e)]))
	    e = G.cyclic_adj_succ(e);
	  dist[u][v] += pos[G.source(e)].distance(pos[G.target(e)]);
	  e = G.cyclic_adj_succ(G.reversal(e));
	}
      }
    }
  }
}





